
#include <iostream>
#include <vector>
#include <string>
#include<stdexcept>
#include <fstream>
#include "sudoku_grid.h"
using namespace std;

coords::coords(int x,int y):xy(vector<int>(2,0))
{
	xy[0]=x;
	xy[1]=y;
}
int & coords::operator[](int i)
{
	if(i!=0 && i!=1)
		throw out_of_range("coords holds just 2 values");
	return xy[i];
}

int coords::operator[](int i) const
{
	if(i!=0 && i!=1)
		throw out_of_range("coords holds just 2 values");
	return xy[i];
}
bool coords::operator==(coords rhs) const
{
	if (xy[0]==rhs.xy[0] && xy[1]==rhs.xy[1])
		return true;
	return false;
}
bool coords::operator!=(coords rhs) const
{
	return !(*this==rhs);//'this' is a pointer to the calling object
}





sudoku_grid::sudoku_grid()
{
	vector<int> square;
	for(int i=1; i<=9; i++)
		square.push_back(i);
	vector<vector<int> > row;
	for(int i=0; i<9; i++)
		row.push_back(square);
	for(int i=0; i<9; i++)
	grid.push_back(row);
	
}


sudoku_grid::sudoku_grid(string file_name)
{
	vector<int> square;
	for(int i=1; i<=9; i++)
		square.push_back(i);
	vector<vector<int> > row;
	for(int i=0; i<9; i++)
		row.push_back(square);
	for(int i=0; i<9; i++)
	grid.push_back(row);
	
	ifstream gamefile(file_name.c_str());
	if(!gamefile)
		cout<<"\nUnable to open "<<file_name<<endl;
	coords curr_square;
	int x;
	while(gamefile>>x && curr_square[0]!=9)
	{
		if(x>=1&&x<=9)
			set(curr_square,x);
		curr_square[1]=(curr_square[1]+1)%9;
		if(curr_square[1]==0)
			curr_square[0]++;
	}
	gamefile.clear();

}
	
	
void sudoku_grid::display() const
{
cout<<endl<<string(73,'_')<<endl;
	for(int i=0; i<9; i++)
	{
		for(int base=0;base!=9; base+=3)
		{
			cout<<"| ";
				for(int j=0;j<9;j++)
				{
					for(int r=0; r<3;r++)
					{
					(grid[i][j][base+r]==0)?cout<<"  ":
						cout<<grid[i][j][base+r]<<" ";
					}
				(j%3==2)?cout<<"| ":cout<<". ";
			}
			cout<<endl;
		}
		(i%3==2)? cout<<string(73,'_')<<endl:cout<<string(73,'.')<<endl;
	}
	return;
}


bool sudoku_grid::set(coords square, int value)
{
	//Test that the value is in the square.
	if (grid[square[0]][square[1]][value-1]!=value)
	{
		cout<<"\nThe value "<<value<<" is not available for "
			<<"square ("<<square[0]<<", "<<square[1]
			<<")\n";
		return false;
	}
	// Delete all other values from the square.
	for(int other_val=1; other_val<=9;other_val++)
	{
		if(other_val!=value && grid[square[0]][square[1]][other_val-1]==other_val)
		{
			if(!del(square,other_val))
				return false;
		}
	}
	// Delete the value to set from the rest of the 
	// row, column, and block.		
	coords new_square(square[0],0);
	//Remove from row
	for(int col=0;col!=9;col++)
	{
		if(col!=square[1]&&grid[square[0]][col][value-1]==value)
		{
			new_square[1]=col;
			if(!(del(new_square,value)))
				return false;
		}
	}
	new_square[1]=square[1];
	//Remove from column
	for(int row=0;row!=9;row++)
	{
		if(row!=square[0]&&grid[row][square[1]][value-1]==value)
		{
			new_square[0]=row;
			if(!(del(new_square,value)))
				return false;
		}
	}
	//Remove from block
	int row_base=square[0]-square[0]%3;
	int col_base=square[1]-square[1]%3;
	for(int row_r=0;row_r<3;row_r++)
		for(int col_r=0;col_r<3;col_r++)
		{
			new_square[0]=row_base+row_r;
			new_square[1]=col_base+col_r;
			if(new_square!=square && grid[new_square[0]][new_square[1]][value-1]==value)
				if(!(del(new_square,value)))
					return false;
		}
			
	return true;
}

/* Delete the specified value from the given square,
 * unless that leaves the square empty.
 */
 
bool sudoku_grid::del(coords square, int value)
{
	grid[square[0]][square[1]][value-1]=0;
	vector<int> count=value_count(square);
	if(count[0]==0)
	{
		cout<<"\nNo possibilities remain in "
			<<"square ("<<square[0]<<", "<<square[1]
			<<")\n";
		return false;
	}
	
	return true;
}

vector<int> sudoku_grid::value_count(coords square) const
{
	vector<int> ret_vec(2,0);
	for(int val=1;val<=9;val++)
		if(grid[square[0]][square[1]][val-1]==val)
		{
			ret_vec[0]++;
			ret_vec[1]=val;
		}
	return ret_vec;
}


















